function Fit_growth_rate
% Function that fits the growth rate (from data) to a model describing the
% growth rate in time.
% Abbreviations: OD = OD600
clear all; clc; close all
t_min                       = 1;                                    
t_max                       = 64;
filename                    = 'Data_kinetic_experiment.xlsx';
num                         = xlsread(filename);                    
time                        = num(2,t_min:t_max)./3600;             % time [h]
tData                       = time;                                 % time [h]    
OD_matrix                   = num(4:27,t_min:t_max);                % Matrix with all the OD600 measurements
OD_rh_2_wt                  = OD_matrix(5:7,:);                     % OD600 measurement for all 3 experiments with 0.2% (w/v) Rhamnose
mean_OD_rh_2_wt             = mean(OD_rh_2_wt);                     % Average of the OD600 for the experiment of 0.2% (w/v) Rhamnose

%% Calculate the growth rate (mu) from the data
for i = t_min:t_max - 1
    mu_rh_2_wt(i)           = log((mean_OD_rh_2_wt(1,i+1))...
        ./(mean_OD_rh_2_wt(1,i)))./(time(i+1) - time(i));           % Growth rate mu derived from data points
    time_av(i)              = (time(1+i) + time(i))./2;             % Average time interval
end
% After about 10 hours, the growth rate becomes 0. Due to the error of the measurement, the growth rate can become slightly negative. 
% Correct for this by setting the growth rate to 0 when the growth rate is close to 0. 
time_av(2)                  = [];
mu_rh_2_wt(2)               = [];
mu_rh_2_wt(45:end)          = 0;

%% MODEL 1: model without lag time included
ODData                      = mean_OD_rh_2_wt;
p0_1                        = [0.2; 2];
p_1                         = lsqcurvefit(@fun, p0_1, tData, ODData, [0, 0], [2, 2])

assignin('base','p_1',p_1); assignin('base','ODData',ODData);

mu_max_1                    = p_1(1);                               % Maximum growth rate calculated for model 1
OD_max_1                    = p_1(2);                               % Maximum OD600 calculated for model 1
OD_0                        = 0.0616;                               % Initial OD600 calculated (true for both model 1 and 2)
OD_fitted_1                 = OD_max_1.*OD_0.*exp(mu_max_1.*tData)...
                 ./(OD_max_1 + OD_0.*(exp(mu_max_1.*tData) - 1));   % OD fitted for model 1

growth_rate_1               = mu_max_1.*(1 - OD_fitted_1./OD_max_1);% Growth rate fitted for model 1

%% MODEL 2: model without lag time included
p0_2                        = [0.28; 1; 0.2];
p_2                         = lsqcurvefit(@fun_2, p0_2, tData, ODData, [0, 0, 0], [2, 2, 0.2])

mu_max_2                    = p_2(1);                               % Maximum growth rate calculated for model 2
OD_max_2                    = p_2(2);                               % Maximum OD calculated for model 2
q_0                         = p_2(3);                               % Parameter characterizing the lag time for model 2. 
z                           = (OD_0  ./ (OD_max_2 - OD_0 )) .* ( 1 + q_0.*exp(mu_max_2  .* tData) ) ./ (1 + q_0);
OD_fitted_2                 = OD_max_2.* (1 - 1./(z + 1) );         % OD fit function for model 1

assignin('base','p_2',p_2);

growth_rate_2               = (1./OD_max_2).*(1./(1 + (1./q_0)...
    .*exp(-mu_max_2.*tData))).*mu_max_2.*(1-OD_fitted_2./OD_max_2); % Growth rate fitted for model 2
assignin('base','growth_rate_2',growth_rate_2);
%% Plot the figures

% OD fitted as a function of time for model 1. Data points are shown as well as the model.
figure(1) 
set(gca,'fontsize',20)
plot(time,mean_OD_rh_2_wt,'ro','LineWidth',4); hold on;
plot(time,OD_fitted_1,'b','LineWidth',4);
legend('OD_{600} data','OD_{fitted} for model 1');
xlabel('Time [h]'); ylabel('OD_{600}');

% Growth rate as a function of time for model 1. 
figure(2)
set(gca,'fontsize',20)
plot(time,growth_rate_1,'LineWidth',4);hold on;
plot(time_av,mu_rh_2_wt,'ro','LineWidth',4);
xlabel('Time [h]'); ylabel('Growth rate \mu [1/h]');
legend('\mu_{data}','\mu_{fitted} with model 1 without lag time');

% OD fitted as a function of time for model 2. Data points are shown as well as the model.
figure(3) 
set(gca,'fontsize',20)
plot(time,mean_OD_rh_2_wt,'ro','LineWidth',4); hold on;
plot(time,OD_fitted_2,'b','LineWidth',4);
legend('OD_{600} data','OD_{fitted} for model 2');
xlabel('Time [h]'); ylabel('OD_{600}');

% Growth rate as a function of time for model 2. 
figure(4)
set(gca,'fontsize',20)
plot(time,growth_rate_2,'LineWidth',4);hold on;
plot(time_av,mu_rh_2_wt,'ro','LineWidth',4); hold on
xlabel('Time [h]'); ylabel('Growth rate \mu [1/h]');
legend('\mu_{data}','\mu_{fitted} with model 2 including lag time');

%% Statistical analysis
% MODEL 1
SSE_1                     = sum((ODData - OD_fitted_1).^2);
SST_1                     = sum((ODData - mean(ODData)).^2);
R_squared_1               = 1-(SSE_1/SST_1)                         % R squared value for the fit with model 1

% MODEL 2
SSE_2                     = sum((ODData - OD_fitted_2).^2);
SST_2                     = sum((ODData - mean(ODData)).^2);
R_squared_2               = 1-(SSE_2/SST_2)                         % R squared value for the fit with model 2


end
function OD_fitted_1 = fun( p_1,tData) % function that has to be fitted
% Fit with the data from the experiment with 0.2% (w/v) Rhamnose
mu_max_1                    = p_1(1);                               % Maximum growth rate calculated for model 1
OD_max_1                    = p_1(2);                               % Maximum OD600 calculated for model 1
OD_0                        = 0.0616;                               % Initial OD600 calculated (true for both model 1 and 2)
OD_fitted_1 = OD_max_1 .*OD_0 .*exp(mu_max_1.*tData)./(OD_max_1  + ...
    OD_0 .*(exp(mu_max_1.*tData) - 1));                             % OD fit function for model 1
end

function OD_fitted_2 = fun_2( p_2,tData) % function that has to be fitted
% Fit with the data from the experiment with 0.2% (w/v) Rhamnose
mu_max_2                    = p_2(1);                               % Maximum growth rate calculated for model 2
OD_max_2                    = p_2(2);                               % Maximum OD calculated for model 2
q_0                         = p_2(3);                               % Parameter characterizing the lag time for model 2. 
OD_0                        = 0.0616;                               % Initial OD600 calculated (true for both model 1 and 2)
z                           = (OD_0  ./ (OD_max_2 - OD_0 )) .* ( 1 + q_0.*exp(mu_max_2  .* tData) ) ./ (1 + q_0);
OD_fitted_2                 = OD_max_2.* (1 - 1./(z + 1) );         % OD fit function for model 1
end
