function Fit_kinetic_model_rhamnose
clear all; close all; clc
% Model that calculates the immature/mature GFP protein concentrations in
% time. Furthermore, the csgA transport rate is estimated.
% GFPmut3: http://parts.igem.org/wiki/index.php?title=Part:BBa_E0040 (GFP of part I13504)
% https://greenfluorescentblog.wordpress.com/tag/gfpmut3/           GFP maturation
filename = 'Data_kinetic_experiment.xlsx';
num = xlsread(filename);
%% Variables
t_min               = 1;
t_max               = 83;                                           % Number of time points
OD_600              = 8e8;                                          % Amount of cells corresponding to an OD600 of 1
N_av                = 6.022e23;                                     % Number of Avogadro [molecules per mole]
mol_GFP             = 27e3;                                         % Molecular weight of GFP [g/mol]
OD_matrix           = num(4:27,t_min:t_max);                        % Matrix with all OD600 measurements
GFP_488_matrix      = num(31:54,t_min:t_max);                       % Matrix with all fluoresence measurements (excitation at 488nm)
time                = num(2,t_min:t_max)./3600;                     % Vector with all time points
time                = time - time(1);                               % Put t = 4 hours to t = 0 hours, as promoter is active after 4 hours
assignin('base','time',time);

F_OD_rh_2_wt_GFP    = (GFP_488_matrix(5:7,:) - ...                  % Fluorescence divided by the OD600, csgA-GFPmut3 construct with 0.2% (w/v) rhamnose
    (GFP_488_matrix(8,1)))./OD_matrix(5:7,:);                       % (corrected for background fluorescence)
F_OD_rh_5_wt_GFP    = (GFP_488_matrix(9:11,:) - ...                 % Fluorescence divided by the OD600, csgA-GFPmut3 construct with 0.5% (w/v) rhamnose
    (GFP_488_matrix(12,1)))./OD_matrix(9:11,:);                     % (corrected for background fluorescence)

mean_rh_2_wt_GFP    = mean(F_OD_rh_2_wt_GFP);                       % mean of the fluorescence divided by the OD600, csgA-GFPmut3 construct with 0.2% (w/v) rhamnose
mean_rh_5_wt_GFP    = mean(F_OD_rh_5_wt_GFP);                       % mean of the fluorescence divided by the OD600, csgA-GFPmut3 construct with 0.5% (w/v) rhamnose

Mass_rh_2_wt_GFP    = calibration(mean_rh_2_wt_GFP);                % Amount of GFPmut3 in nanogram per OD600 for an induction level of 0.2% (w/v) rhamnose
Mass_rh_5_wt_GFP    = calibration(mean_rh_5_wt_GFP);                % Amount of GFPmut3 in nanogram per OD600 for an induction level of 0.5% (w/v) rhamnose

Mass_rh_2_wt_GFP    = (Mass_rh_2_wt_GFP./OD_600).*1e-9;             % Amount of GFPmut3 in gram per cell for an induction level of 0.2% (w/v) rhamnose
Mass_rh_5_wt_GFP    = (Mass_rh_5_wt_GFP./OD_600).*1e-9;             % Amount of GFPmut3 in gram per cell for an induction level of 0.5% (w/v) rhamnose

Molec_rh_2_wt_GFP   = (Mass_rh_2_wt_GFP./mol_GFP).*N_av;            % Amount of GFPmut3 in molecules per cell for an induction level of 0.2% (w/v) rhamnose
Molec_rh_5_wt_GFP   = (Mass_rh_5_wt_GFP./mol_GFP).*N_av;            % Amount of GFPmut3 in molecules per cell for an induction level of 0.5% (w/v) rhamnose

SST_rh_2_wt_GFP     = Molec_rh_2_wt_GFP(end)                        % Steady state concentration of GFPmut3 in molecules per cell for an induction level of 0.2% (w/v) rhamnose
SST_rh_5_wt_GFP     = Molec_rh_5_wt_GFP(end)                        % Steady state concentration of GFPmut3 in molecules per cell for an induction level of 0.5% (w/v) rhamnose

Ratio_SST_concen    = SST_rh_5_wt_GFP./ SST_rh_2_wt_GFP             % Ratio of steady state concentrations of GFPmut3 (0.5% (w/v) rhamnose : 0.2% (w/v) rhamnose)
%% Call ODE
% n_hours = max(time);                                              % Maximum number of hours
n_hours=24;
t_vec = linspace(0,n_hours.*3600);                                  % Time vector [s]
C_0 = [0 0 0 0.151./3600 0 0 0];                                        % Initial values of the system of ordinary differential equations (ODE's)

C = [];
PoPs_vec = [1.58e-04./(Ratio_SST_concen) ; 1.58e-04]
%% Calculate the csgA transport rate
mu                  = 0;                                            % Growth rate before measurent of intracellular csgA concentration
y_P                 = 6.9.*10^-5;                                   % Protein degradation rate [1/s]
y_M                 = 4.8e-3;                                       % mRNA degradation rate [1/s]
n                   = 200;                                          % csgA plasmid copy number 
M_csgA_SS           = n.*PoPs_vec./y_M;                             % Steady state csgA mRNA concentration [molecules/cell]


P_csgA_In_SS        = 0.5.*[SST_rh_2_wt_GFP;SST_rh_5_wt_GFP]        % Steady state csgA intracellular concentration [molecules/cell]
k_tr                = 0.4.*(238./151);                              % Translation rate csgA [proteins/s/mRNA]
k_t                 = k_tr.* (M_csgA_SS ./P_csgA_In_SS) - y_P - mu  % csgA transport rate [1/s]

R_csgA_SS           = k_t.*P_csgA_In_SS
%% Call the ODE
for i = 1:2
    [t,P] = ode45(@(t,C) ode_gene(t,C,PoPs_vec(i),k_t(i)), t_vec , C_0 );  % Call the system of ODE's
    C = [C;P];
end

M                   = C(:,1);                                       % mRNA per cell of GFPmut3
I                   = C(:,2);                                       % GFPmut3 immature protein concentration in molecules per cell
G                   = C(:,3);                                       % GFPmut3mature protein concentration in molecules per cell
mu                  = C(:,4);                                       % Growth rate
M_csgA              = C(:,5);                                       % csgA mRNA concentration
P_csgA_in           = C(:,6);                                       % Intracellular csgA concentration                                
P_csgA_ex           = C(:,7);                                       % Extracellular csgA concentration    
assignin('base','C',C);

%% Figures
figure(1)
set(gca,'fontsize',20)
plot(time, Molec_rh_2_wt_GFP,'r--','LineWidth',4); hold on;
plot(time, Molec_rh_5_wt_GFP,'b--','LineWidth',4); hold on
legend('Rhamnose 0.2% (w/v) csgA-GFPmut3','Rhamnose 0.5% (w/v) csgA-GFPmut3','Location','NorthWest'); hold on;
ylim([-0.1e4 3.5e4]); xlim([0 n_hours]);
xlabel('Time [h]'); ylabel('GFPmut3 [molecules/cell]');

figure(2)
set(gca,'fontsize',20)
plot(time, Molec_rh_2_wt_GFP,'r--','LineWidth',4); hold on;
plot(t_vec./3600,G(1:100),'b','LineWidth',4); hold on
ylim([-0.05e4 2e4]); xlim([0 n_hours]);
legend('Data   Rhamnose 0.2% (w/v) csgA-GFPmut3','Model Rhamnose 0.2% (w/v) csgA-GFPmut3','Location','NorthWest'); hold on;
xlabel('Time [h]'); ylabel('GFPmut3 [molecules/cell]');

figure(3)
set(gca,'fontsize',20)
plot(time, Molec_rh_5_wt_GFP,'r--','LineWidth',4); hold on;
plot(t_vec./3600,G(101:200),'b','LineWidth',4); hold on
ylim([-0.05e4 4e4]); xlim([0 n_hours]);
legend('Data   Rhamnose 0.5% (w/v) csgA-GFPmut3','Model Rhamnose 0.5% (w/v) csgA-GFPmut3','Location','NorthWest'); hold on;
xlabel('Time [h]'); ylabel('GFPmut3 [molecules/cell]');

figure(4)
set(gca,'fontsize',20)
plot(t_vec./3600,P_csgA_ex(1:100),'r','LineWidth',4); hold on
plot(t_vec./3600,P_csgA_ex(101:200),'b','LineWidth',4); hold on
legend('csgA^{Extracellular} Rhamnose 0.2% (w/v)','csgA^{Extracellular} Rhamnose 0.5% (w/v)','Location','NorthWest'); hold on;
xlabel('Time [h]'); ylabel('csgA^{Extracellular} [molecules/cell]');

%% Check the intracellular concentration of GFP
% figure(5)
% subplot(2,1,1)
% plot(t_vec./3600,I(1:100),'r','LineWidth',4); hold on
% plot(t_vec./3600,I(101:200),'b','LineWidth',4); hold on

end

function Mass_GFP   = calibration(Fluorescence)
m                   = 59.7;                                         % Slope of calibration curve
Mass_GFP            = Fluorescence./m;                              % Amount of GFPmut3 in nanogram
end

function dCdt = ode_gene(t,C,PoPs_vec,k_t)

mu_max                      = 0.9648./3600;                         % Maximum growth rate calculated for model 2
OD_max                      = 1.0074;                               % Maximum OD calculated for model 2
q_0                         = 0.2000;                               % Parameter characterizing the lag time for model 2.
OD_0                        = 0.0616;                               % Initial OD600 calculated (true for both model 1 and 2)
n_eq                        = 7;                                    % Number of equation
if t < (3.5.*3600);
    PoPs                        = 0;                                % Promoter is active after 4 hours
else
    PoPs                    = PoPs_vec;                             % Activity of the promoter after 4 hours (assumption: promoter at its maximum activity)
end

n                           = 200;                                  % csgA plasmid copy number

y_M                         = 4.8e-3;                               % mRNA degradation rate [1/s]
k_tr                        = 0.4;                                  % translation rate of GFP [proteins/s/mRNA]
k_M                         = 6.74e-04;                             % GFP maturation rate
y_P                         = 6.9.*10^-5;                           % Immature protein degradation rate

%% constants for csgA modelling
k_tr_csgA                   = 0.63;                                 % translation rate of csgA [proteins/s/mRNA]

M                           = C(1);                                 % GFP mRNA concentration
I                           = C(2);                                 % GFP Immature protein concentration
G                           = C(3);                                 % GFP Mature protein concentration
mu                          = C(4);                                 % Growth rate of our culture
M_csgA                      = C(5);                                 % csgA mRNA concentration
P_csgA_in                   = C(6);                                 % Intracellular csgA concentration                                
P_csgA_ex                   = C(7);                                 % Extracellular csgA concentration    



dCdt                        = zeros(n_eq,1);

dCdt(1)                     = n.*PoPs - y_M.*M;                     % mRNA concentration as a function of time
dCdt(2)                     = k_tr.*M - k_M.*I - mu.*I - y_P.*I;    % Immature protein concentration as a function of time
dCdt(3)                     = k_M.*I - mu.*G - y_P.*G;              % Mature protein concentration as a function of time

%% Growth rate ODE
f                           = mu_max./(q_0 + exp(-mu_max.*t));
z                           = (OD_0./(OD_max - OD_0)).*( 1 + q_0.*exp(mu_max.*t))./(1 + q_0);
N                           = OD_max .* (1 - 1./(z + 1));
f_ac                        = f.^2 .* exp(- mu_max.*t);
g                           = (q_0./OD_max).*(OD_max - N);

g_ac                        = -( (q_0)./( 1 + q_0) ).*(OD_0./(OD_max - OD_0)).*(1./(z + 1).^2).*mu_max.*q_0.*exp(mu_max.*t);
dCdt(4)                     = f_ac.*g + f.*g_ac;                    % Growth rate as a function of time

%% csgA model

dCdt(5)                     = n.*PoPs - y_M.*M_csgA ;
dCdt(6)                     = k_tr_csgA.*M_csgA - y_P.*P_csgA_in - mu.*P_csgA_in - k_t.*P_csgA_in;
dCdt(7)                     = k_t.*P_csgA_in;
end




